function depthPlots()
% creates plots of max acceleration, displacement, and shear stress/strain with depth
close all

% load acceleration data
[a, nNode, nStep, allTimes] = getAccel();

% load data
dis = load('displacement.out');

% remove time column from data
dis(:,1) = [];
% don't consider gravity loading steps
dis = dis(51:end,:);

% reshape data
u = reshape(dis, nStep, 2, nNode);

% load strain and stress data at gauss point 1
strain1 = load('strain1.out');
stress1 = load('stress1.out');

% remove time column from data
strain1(:,1) = [];
stress1(:,1) = [];
% don't consider gravity loading steps
strain1 = strain1(51:end,:);
stress1 = stress1(51:end,:);

[nStep, nStress] = size(stress1);
nRecord = 5;
nElem   = nStress/nRecord;

% reshape stress/strain data
strain = reshape(strain1, nStep, 3, nElem);
stress = reshape(stress1, nStep, 5, nElem);

% determine maximum responses at each depth
aMax = zeros(nNode,1);
uMax = aMax;
tMax = zeros(nElem,1);
gMax = tMax;
% loop over nodes
for k = 1:2:nNode
  % horizontal acceleration
    aMax(k) = max(abs(a(:,1,k)));
  % horizontal displacement
    uMax(k) = max(abs(u(:,1,k)));
end
mId = aMax>0;
aMax = aMax(mId);
uMax = uMax(mId);

%loop over elements
for k = 1:nElem
  % shear stress
    tMax(k) = max(abs(stress(:,4,k)));
  % shear strain
    gMax(k) = max(abs(strain(:,3,k)));
end

% vector of nodal depths
nodeDepth = linspace(50,0,nNode/2)';

% vector of elemental depths (use approximation --> center of element)
elemDepth = linspace(49.5,0.5, nElem);

% information to plot layer boundaries
xVal = [-1e6 1e6];
lay1 = [2 2];
lay2 = [10 10];

% plot the responses
figure(1)
subplot(2,2,1)
    hold on
    plot(xVal,lay1,'--k','linewidth',1.5)
    plot(xVal,lay2,'--k','linewidth',1.5)
    plot(aMax,nodeDepth,'linewidth',1.5)
    hold off
    grid on
    box on
    axis([0 0.6 0 50])
    xlabel('max acceleration (g)','fontsize',16)
    ylabel('depth (m)','fontsize',16)
    set(gca,'YDir','reverse','fontsize',16)

subplot(2,2,2)
    hold on
    plot(xVal,lay1,'--k','linewidth',1.5)
    plot(xVal,lay2,'--k','linewidth',1.5)
    plot(uMax*1000,nodeDepth,'linewidth',1.5)
    hold off
    grid on
    box on
    axis([110 115 0 50])
    xlabel('max displacement (mm)','fontsize',16)
    set(gca,'YDir','reverse','YTickLabel',[],'fontsize',16)

subplot(2,2,3)
    hold on
    plot(xVal,lay1,'--k','linewidth',1.5)
    plot(xVal,lay2,'--k','linewidth',1.5)
    plot(tMax,elemDepth,'linewidth',1.5)
    hold off
    grid on
    box on
    axis([0 300 0 50])
    xlabel('max shear stress (kPa)','fontsize',16)
    ylabel('depth (m)','fontsize',16)
    set(gca,'YDir','reverse','fontsize',16)

subplot(2,2,4)
    hold on
    plot(xVal,lay1,'--k','linewidth',1.5)
    plot(xVal,lay2,'--k','linewidth',1.5)
    plot(gMax*100,elemDepth,'linewidth',1.5)
    hold off
    grid on
    box on
    axis([0 0.25 0 50])
    xlabel('max shear strain (%)','fontsize',16)
    set(gca,'YDir','reverse','YTickLabel',[],'fontsize',16)

return
